/* This file is part of the KDE project

   Copyright (C) 2004 Gary Cramblitt <garycramblitt@comcast.net>

   Adapted from kdeutils/ark/konqplugin by
        Georg Robbers <Georg.Robbers@urz.uni-hd.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; version 2
   of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "akregatorplugin.h"

#include <kapplication.h>
#include <kmimetype.h>
#include <kdebug.h>
#include <kaction.h>
#include <kinstance.h>
#include <klocale.h>
#include <konq_popupmenu.h>
#include <kpopupmenu.h>
#include <kgenericfactory.h>
#include <kurl.h>
#include <dcopclient.h>
#include <kprocess.h>
#include <khtml_part.h>
#include <khtmlview.h>
#include <dom/dom_element.h>
#include <dom/dom_doc.h>
#include <dom/dom_string.h>
#include <dom/html_misc.h>
#include <kmessagebox.h>

#include <qdir.h>
#include <qcstring.h>
#include <qobject.h>

typedef KGenericFactory<AkregatorMenu, KonqPopupMenu> AkregatorMenuFactory;
K_EXPORT_COMPONENT_FACTORY( libakregatorkonqplugin, AkregatorMenuFactory("akregatorkonqplugin") );

AkregatorMenu::AkregatorMenu( KonqPopupMenu * popupmenu, const char *name, const QStringList& /* list */ )
                : KonqPopupMenuPlugin( popupmenu, name)
{
    if ( QCString( kapp->name() ) == "kdesktop" && !kapp->authorize("editable_desktop_icons" ) )
        return;

    // Do nothing if user has turned us off.
    // TODO: Not yet implemented in aKregator settings.
    m_conf = new KConfig( "akregatorrc" );
    m_conf->setGroup( "AkregatorKonqPlugin" );
    if ( !m_conf->readBoolEntry( "Enable", true ) )
        return;

    KGlobal::locale()->insertCatalogue("akregator_konqplugin");
    // Build list of supported MIME types in m_feedMimeTypes.
    feedMimeTypes();
    // Get the list of URLs clicked on from Konqi.
    m_list = popupmenu->fileItemList();
    // See if any are RSS feeds.
    KFileItemListIterator it( m_list );
    KFileItem * item;
    bool hasFeed = false;

    KHTMLView *v=(KHTMLView *)popupmenu->parent()->qt_cast("KHTMLView");
    if (v)
    {
        KHTMLPart *p=v->part();
	bool clickOnBG=m_list.first()->url()==p->url(); // are we clicking on root?
    	if (clickOnBG)
	{
		DOM::Document d=p->document();
		DOM::NodeList l=d.getElementsByTagName("link");
		DOM::Element e;
		QString rel;
		QString target;
		for (uint i=0; i< l.length(); i++)
		{
			e=l.item(i);
			if (!e.isNull())
			{
				DOM::DOMString rel=e.getAttribute("rel");
				if (rel=="alternate" || rel=="service.feed")
				{
					DOM::DOMString url=e.getAttribute("href").copy();
					if (!url.isEmpty())
					{
                                                kdDebug() << "adding "<<url.string() << endl;
						m_feedList.append(fixRelativeURL(url.string(), p->url()));
						hasFeed=true;
					}
				}
			}
		}
		if (!hasFeed) // if there is no link tag with feeds
			      // check the links in the document
		{
			const DOM::HTMLDocument hd=p->htmlDocument();
			if (!hd.isNull())
			{
				const DOM::HTMLCollection links = hd.links();
				for (uint j=0; j< links.length(); j++)
				{
				        DOM::Element e=links.item(j);
					if (!e.isNull())
					{
						DOM::DOMString url=e.getAttribute("href").copy();
						if (isFeedUrl(url.string()))
						{
							m_feedList.append(fixRelativeURL(url.string(), p->url()));
							hasFeed=true;
						}
					}
				}
			}
		}

	}
    }

    if (!hasFeed)
    {
    	while ( ( item = it.current() ) != 0 )
    	{
        	++it;
		KURL u=item->url(); // uh, this breaks with foo/index.html.. need to fix it
	        if (isFeedUrl(item))
        	{
	            kdDebug() << "Akregator::Menu found feed URL " << item->url().prettyURL() << endl;
	            m_feedList.append(fixRelativeURL(item->url().url(), u));
        	    hasFeed = true;
	            break;
        	}
    	}
    }


    // If any of the URLs are RSS feeds, add entry to Konqi popup menu.
    if (hasFeed)
    {
        KAction * action;
        action = new KAction( i18n( "Add Feed to aKregator" ), "akregator", 0, this,
            SLOT( slotAddFeed() ), actionCollection(), "akregatorkonqplugin_mnu" );
        addAction( action );
        addSeparator();
    }
}

AkregatorMenu::~AkregatorMenu()
{
    delete m_conf;
}

void AkregatorMenu::feedMimeTypes()
{
    m_feedMimeTypes << "text/rss";
    m_feedMimeTypes << "text/rdf";
    m_feedMimeTypes << "text/xml";
}

bool AkregatorMenu::isFeedUrl(const QString &url)
{
    if (url.contains(".htm", false)) return false;
    if (url.contains("rss", false) != 0) return true;
    if (url.contains("rdf", false) != 0) return true;
    if (url.contains("xml", false) != 0) return true;
    return false;
}

// handle all the wild stuff that KURL doesn't handle
QString AkregatorMenu::fixRelativeURL(const QString &s, const KURL &baseurl)
{
	QString s2=s;
	KURL u;
	if (KURL::isRelativeURL(s2))
	{
        	if (s2.startsWith("//"))
        	{
            		s2=s2.prepend(baseurl.protocol()+":");
            		u=s2;
		}
        	else if (s2.startsWith("/"))
        	{
	            	u=baseurl;
        		u.setPath(s2);
        	}
        	else
        	{
            		u=baseurl;
            		u.addPath(s2);
        	}
        	u.cleanPath();
    	}
    	else
        	u=s2;

	u.cleanPath();
	return u.url();
}

bool AkregatorMenu::isFeedUrl(const KFileItem * item)
{
    if ( m_feedMimeTypes.contains( item->mimetype() ) )
        return true;
    else
    {
        QString url = item->url().prettyURL();
        // If URL ends in .htm or .html, it is not a feed url.
        return isFeedUrl(url);
    }
    return false;
}

void AkregatorMenu::slotAddFeed()
{
    // Get the DCOP ID of aKregator application.  Any one will do, take first one found.
    DCOPClient *client = kapp->dcopClient();
    QCStringList dcopClients = client->registeredApplications();
    QCStringList::iterator clientIt;
    QCString akregatorClient;
    for (clientIt = dcopClients.begin(); (clientIt != dcopClients.end()) ; ++clientIt )
    {
        if ((*clientIt).contains("akregator", false) != 0) // FIXME: what happens when there's application named akregatorsomething?
        {
            akregatorClient = *clientIt;
            kdDebug() << "AkregatorMenu::slotAddFeed aKregator DCOP ID is " << akregatorClient << endl;
            break;
        }
    }

    if (akregatorClient.isNull())
    {
        // If aKregator not running, start it using KProcess and pass URLs via command line.
        KProcess *proc = new KProcess;
        *proc << "akregator" << "-g" << i18n("Imported feeds");

        for ( QStringList::Iterator it = m_feedList.begin(); it != m_feedList.end(); ++it ) {
		*proc << "-a" << *it;
	}

        proc->start(KProcess::DontCare);
        delete proc;
    }
    else
    {
        // Locate the object whose name begins with aKregatorPart.  Any one will do, take the first found.
        QCStringList akregatorObjs = client->remoteObjects(akregatorClient);
        QCString akregatorObj;
        for (clientIt = akregatorObjs.begin(); (clientIt != akregatorObjs.end()) ; ++clientIt )
        {
            if ((*clientIt).contains("aKregatorIface", false) != 0)
            {
                akregatorObj = *clientIt;
                kdDebug() << "AkregatorMenu::slotAddFeed aKregator object is " << akregatorObj << endl;
                break;
            }
        }
        if (akregatorObj.isNull())
            return;

        for ( QStringList::Iterator it = m_feedList.begin(); it != m_feedList.end(); ++it ) {
                kdDebug() << "AkregatorMenu::slotAddFeed Telling aKregator to add feed " << *it << endl;
                QByteArray  data;
                QDataStream arg(data, IO_WriteOnly);
                arg << *it;
                arg << i18n("Imported feeds");
                if ( !client->send(akregatorClient, akregatorObj, "addFeedToGroup(QString,QString)",
                    data) )
                KMessageBox::error( 0, i18n( "aKregator Popupmenu Plugin - DCOP Call failed" ),
                    i18n( "The DCOP call addFeedToGroup failed" ));
        }
    }
}

#include "akregatorplugin.moc"
